@@ -7,7 +7,7 @@
 #  Author:     Pavel Michna (rnetcdf-devel@bluewin.ch)
 #              Milton Woods (miltonjwoods@gmail.com)
 #
-#  Copyright:  (C) 2004-2020 Pavel Michna, Milton Woods
+#  Copyright (C) 2004-2023 Pavel Michna and Milton Woods.
 #
 #===============================================================================
 #
@@ -38,8 +38,8 @@
 
 att.copy.nc <- function(ncfile.in, variable.in, attribute, ncfile.out, variable.out) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile.in) == "NetCDF")
-  stopifnot(class(ncfile.out) == "NetCDF")
+  stopifnot(inherits(ncfile.in, "NetCDF"))
+  stopifnot(inherits(ncfile.out, "NetCDF"))
   stopifnot(is.character(attribute) || is.numeric(attribute))
   stopifnot(is.character(variable.in) || is.numeric(variable.in))
   stopifnot(is.character(variable.out) || is.numeric(variable.out))
@@ -58,7 +58,7 @@ att.copy.nc <- function(ncfile.in, variable.in, attribute, ncfile.out, variable.
 
 att.delete.nc <- function(ncfile, variable, attribute) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   stopifnot(is.character(attribute) || is.numeric(attribute))
   
@@ -76,7 +76,7 @@ att.delete.nc <- function(ncfile, variable, attribute) {
 att.get.nc <- function(ncfile, variable, attribute,
                        rawchar = FALSE, fitnum = FALSE) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   stopifnot(is.character(attribute) || is.numeric(attribute))
   stopifnot(is.logical(rawchar))
@@ -100,7 +100,7 @@ att.get.nc <- function(ncfile, variable, attribute,
 
 att.inq.nc <- function(ncfile, variable, attribute) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   stopifnot(is.character(attribute) || is.numeric(attribute))
   
@@ -118,7 +118,7 @@ att.inq.nc <- function(ncfile, variable, attribute) {
 
 att.put.nc <- function(ncfile, variable, name, type, value) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   stopifnot(is.character(name))
   stopifnot(is.character(type) || is.numeric(type))
@@ -139,7 +139,7 @@ att.put.nc <- function(ncfile, variable, name, type, value) {
 
 att.rename.nc <- function(ncfile, variable, attribute, newname) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   stopifnot(is.character(attribute) || is.numeric(attribute))
   stopifnot(is.character(newname))
@@ -157,7 +157,7 @@ att.rename.nc <- function(ncfile, variable, attribute, newname) {
 
 close.nc <- function(con, ...) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(con) == "NetCDF")
+  stopifnot(inherits(con, "NetCDF"))
   
   #-- C function call --------------------------------------------------------
   nc <- .Call(R_nc_close, attr(con, "handle_ptr"))
@@ -207,7 +207,7 @@ create.nc <- function(filename, clobber = TRUE, share = FALSE, prefill = TRUE,
 
 dim.def.nc <- function(ncfile, dimname, dimlength = 1, unlim = FALSE) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(dimname))
   stopifnot(is.numeric(dimlength))
   stopifnot(is.logical(unlim))
@@ -225,7 +225,7 @@ dim.def.nc <- function(ncfile, dimname, dimlength = 1, unlim = FALSE) {
 
 dim.inq.nc <- function(ncfile, dimension) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(dimension) || is.numeric(dimension))
   
   #-- C function call --------------------------------------------------------
@@ -243,7 +243,7 @@ dim.inq.nc <- function(ncfile, dimension) {
 
 dim.rename.nc <- function(ncfile, dimension, newname) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(dimension) || is.numeric(dimension))
   stopifnot(is.character(newname))
   
@@ -260,7 +260,7 @@ dim.rename.nc <- function(ncfile, dimension, newname) {
 
 file.inq.nc <- function(ncfile) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   
   #-- C function call --------------------------------------------------------
   nc <- .Call(R_nc_inq_file, ncfile)
@@ -345,7 +345,7 @@ print_grp <- function(x, level = 0) {
 
   #-- Inquire about the group ------------------------------------------------
   grpinfo <- try(grp.inq.nc(x, ancestors = FALSE), silent = TRUE)
-  if (class(grpinfo) == "try-error" || is.null(grpinfo)) {
+  if (inherits(grpinfo, "try-error") || is.null(grpinfo)) {
     return(invisible(NULL))
   }
   
@@ -458,7 +458,7 @@ print_grp <- function(x, level = 0) {
 
 print.nc <- function(x, ...) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(x) == "NetCDF")
+  stopifnot(inherits(x, "NetCDF"))
 
   cat("netcdf ", file.inq.nc(x)$format, " {\n", sep="")
 
@@ -477,7 +477,7 @@ print.nc <- function(x, ...) {
 
 sync.nc <- function(ncfile) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   
   #-- C function call --------------------------------------------------------
   nc <- .Call(R_nc_sync, ncfile)
@@ -493,10 +493,10 @@ sync.nc <- function(ncfile) {
 var.def.nc <- function(ncfile, varname, vartype, dimensions,
                        chunking=NA, chunksizes=NULL,
                        deflate=NA, shuffle=FALSE, big_endian=NA,
-                       fletcher32=FALSE, filter_id=NA,
-                       filter_params=integer(0)) {
+                       fletcher32=FALSE, filter_id=integer(0),
+                       filter_params=list()) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(varname))
   stopifnot(is.character(vartype) || is.numeric(vartype))
 
@@ -511,8 +511,10 @@ var.def.nc <- function(ncfile, varname, vartype, dimensions,
   stopifnot(is.logical(shuffle))
   stopifnot(is.logical(big_endian))
   stopifnot(is.logical(fletcher32))
-  stopifnot(isTRUE(is.na(filter_id)) || is.numeric(filter_id))
-  stopifnot(is.integer(filter_params))
+  stopifnot(is.numeric(filter_id))
+  stopifnot(is.list(filter_params))
+  stopifnot(all(sapply(filter_params, FUN=is.numeric)))
+  stopifnot(length(filter_id) == length(filter_params))
 
   #-- C function call --------------------------------------------------------
   nc <- .Call(R_nc_def_var, ncfile, varname, vartype, dimensions,
@@ -530,7 +532,7 @@ var.get.nc <- function(ncfile, variable, start = NA, count = NA, na.mode = 4,
   collapse = TRUE, unpack = FALSE, rawchar = FALSE, fitnum = FALSE,
   cache_bytes=NA, cache_slots=NA, cache_preemption=NA) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   stopifnot(is.numeric(start) || is.logical(start))
   stopifnot(is.numeric(count) || is.logical(count))
@@ -592,7 +594,7 @@ var.get.nc <- function(ncfile, variable, start = NA, count = NA, na.mode = 4,
 
 var.inq.nc <- function(ncfile, variable) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   
   #-- C function call --------------------------------------------------------
@@ -618,7 +620,7 @@ var.inq.nc <- function(ncfile, variable) {
 
 var.par.nc <- function(ncfile, variable, access="NC_COLLECTIVE") {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   stopifnot(is.character(access) &&
             access[1] %in% c("NC_COLLECTIVE", "NC_INDEPENDENT"))
@@ -638,7 +640,7 @@ var.put.nc <- function(ncfile, variable, data, start = NA, count = NA,
   na.mode = 4, pack = FALSE,
   cache_bytes=NA, cache_slots=NA, cache_preemption=NA) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   stopifnot(is.numeric(data) || is.character(data) || is.raw(data) ||
             is.logical(data) || is.list(data) || is.factor(data))
@@ -761,7 +763,7 @@ var.put.nc <- function(ncfile, variable, data, start = NA, count = NA,
 
 var.rename.nc <- function(ncfile, variable, newname) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(variable) || is.numeric(variable))
   stopifnot(is.character(newname))
   
@@ -778,7 +780,7 @@ var.rename.nc <- function(ncfile, variable, newname) {
 
 grp.def.nc <- function(ncid, grpname) {
   # Check arguments:
-  stopifnot(class(ncid) == "NetCDF")
+  stopifnot(inherits(ncid, "NetCDF"))
   stopifnot(is.character(grpname))
   
   # C function call:
@@ -796,7 +798,7 @@ grp.def.nc <- function(ncid, grpname) {
 
 grp.find <- function(ncid, grpname, full = isTRUE(grepl("/", grpname))) {
   # Check arguments:
-  stopifnot(class(ncid) == "NetCDF")
+  stopifnot(inherits(ncid, "NetCDF"))
   stopifnot(is.character(grpname))
   stopifnot(is.logical(full))
   
@@ -815,7 +817,7 @@ grp.find <- function(ncid, grpname, full = isTRUE(grepl("/", grpname))) {
 
 grp.inq.nc <- function(ncid, grpname = NULL, ancestors = TRUE) {
   # Check arguments:
-  stopifnot(class(ncid) == "NetCDF")
+  stopifnot(inherits(ncid, "NetCDF"))
   stopifnot(is.logical(ancestors))
   stopifnot(is.null(grpname) || is.character(grpname))
   
@@ -877,7 +879,7 @@ grp.inq.nc <- function(ncid, grpname = NULL, ancestors = TRUE) {
 
 grp.rename.nc <- function(ncid, newname, oldname = NULL) {
   # Check arguments:
-  stopifnot(class(ncid) == "NetCDF")
+  stopifnot(inherits(ncid, "NetCDF"))
   stopifnot(is.character(newname))
   stopifnot(is.null(oldname) || is.character(oldname))
   
@@ -899,7 +901,7 @@ grp.rename.nc <- function(ncid, newname, oldname = NULL) {
 
 read.nc <- function(ncfile, recursive = FALSE, ...) {
   #-- Check args -------------------------------------------------------------
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.logical(recursive))
   
   #-- Initialise storage -----------------------------------------------------
@@ -943,7 +945,7 @@ read.nc <- function(ncfile, recursive = FALSE, ...) {
 type.def.nc <- function(ncfile, typename, class, size=NULL, basetype=NULL,
                         names=NULL, values=NULL, subtypes=NULL, dimsizes=NULL) {
   # Check arguments
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.character(typename))
   stopifnot(is.character(class))
   if (isTRUE(class == "compound")) {
@@ -974,7 +976,7 @@ type.def.nc <- function(ncfile, typename, class, size=NULL, basetype=NULL,
 
 type.inq.nc <- function(ncfile, type, fields=TRUE) {
   # Check arguments:
-  stopifnot(class(ncfile) == "NetCDF")
+  stopifnot(inherits(ncfile, "NetCDF"))
   stopifnot(is.numeric(type) || is.character(type))
   stopifnot(is.logical(fields))
   
@@ -1006,15 +1008,12 @@ utcal.nc <- function(unitstring, value, type = "n") {
     colnames(ut) <- c("year", "month", "day", "hour", "minute", "second")
     return(ut)
   } else if (isTRUE(type == "s")) {
-    x <- apply(ut, 1, function(x) {
-      paste(x[1], "-", sprintf("%02g", x[2]), "-", sprintf("%02g", 
-        x[3]), " ", sprintf("%02g", x[4]), ":", sprintf("%02g", x[5]), 
-        ":", sprintf("%02g", x[6]), sep = "")
-    })
+    x <- sprintf("%02g-%02g-%02g %02g:%02g:%02g",ut[,1],ut[,2],ut[,3],ut[,4],ut[,5],ut[,6])
     return(x)
   } else if (isTRUE(type == "c")) {
-    ct <- as.POSIXct(utinvcal.nc("seconds since 1970-01-01 00:00:00 +00:00", 
-      ut), tz = "UTC", origin = ISOdatetime(1970, 1, 1, 0, 0, 0, tz = "UTC"))
+    ct <- utinvcal.nc("seconds since 1970-01-01 00:00:00 +00:00", ut)
+    attr(ct, "class") <- c("POSIXct","POSIXt")
+    attr(ct, "tzone") <- "UTC"
     return(ct)
   }
 }
